@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-import matplotlib.pyplot as plt
+import warnings
+
 import numpy as np
-from scipy.constants import pi
 from scipy.spatial.distance import pdist
 
 from .utilities import _initialize_figure, _format_axes, _format_axes_3d
@@ -693,6 +693,11 @@ def rectangle_field(N_1, N_2, B_1, B_2, H, D, r_b, tilt=0., origin=None):
      0   1   2
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.rectangle_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if origin is None:
@@ -782,6 +787,11 @@ def staggered_rectangle_field(
      0    1    2
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.staggered_rectangle_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if N_1 == 1 or N_2 == 1:
@@ -880,6 +890,11 @@ def dense_rectangle_field(
      0    1    2
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.dense_rectangle_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     if N_1 == 1:
         # line field
         return rectangle_field(N_1, N_2, B, B, H, D, r_b, tilt, origin)
@@ -939,6 +954,11 @@ def L_shaped_field(N_1, N_2, B_1, B_2, H, D, r_b, tilt=0., origin=None):
      0   1   2
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.L_shaped_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if origin is None:
@@ -1026,6 +1046,11 @@ def U_shaped_field(N_1, N_2, B_1, B_2, H, D, r_b, tilt=0., origin=None):
      0   1   2
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.U_shaped_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if origin is None:
@@ -1129,6 +1154,11 @@ def box_shaped_field(N_1, N_2, B_1, B_2, H, D, r_b, tilt=0, origin=None):
      0   1   2   3
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.box_shaped_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if origin is None:
@@ -1240,6 +1270,11 @@ def circle_field(N, R, H, D, r_b, tilt=0., origin=None):
            6
 
     """
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.circle_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
     borefield = []
 
     if origin is None:
@@ -1251,8 +1286,8 @@ def circle_field(N, R, H, D, r_b, tilt=0., origin=None):
         x0, y0 = origin
 
     for i in range(N):
-        x = R * np.cos(2 * pi * i / N)
-        y = R * np.sin(2 * pi * i / N)
+        x = R * np.cos(2 * np.pi * i / N)
+        y = R * np.sin(2 * np.pi * i / N)
         orientation = np.arctan2(y - y0, x - x0)
         # The borehole is inclined only if it does not lie on the origin
         if np.sqrt((x - x0)**2 + (y - y0)**2) > r_b:
@@ -1345,7 +1380,13 @@ def visualize_field(
         Figure object (matplotlib).
 
     """
-    from mpl_toolkits.mplot3d import Axes3D
+    # This function is deprecated as of v2.3. It will be removed in v3.0.
+    warnings.warn("`pygfunction.boreholes.visualize_field` is "
+                  "deprecated as of v2.3. It will be removed in v3.0. "
+                  "Use the `pygfunction.borefield.Borefield` class instead.",
+                  DeprecationWarning)
+
+    from ._mpl import plt
 
     # Configure figure and axes
     fig = _initialize_figure()
